# This example shows a window with 5 elements:
# - A dropdown list, named "com_port", with different COM ports.
# - A dropdown list, named "test", with different tests.
# - A button, named "exec_one" (with same text), to execute only one test.
# - A button, named "exec_all"(with same text), to execute all tests.
# - A label, named "result"(with same text) with the test/s result.
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
import sys
import serial
import glob
import time
import threading
import queue
import os


class MainWindow(QMainWindow):

    def __init__(self, parent=None):
        super(MainWindow, self).__init__(parent)

        # Create the main user interface
        self.centralWidget = QWidget()
        self.setCentralWidget(self.centralWidget)
        self.mainLayout = QVBoxLayout()
        self.centralWidget.setLayout(self.mainLayout)

        # Create the top part with the serial port and test selection
        self.comPortLabel = QLabel("COM Port")
        self.comPortLabel.setAlignment(Qt.AlignCenter)

        self.com_port = QComboBox()
        self.com_port.addItems(self._get_available_ports())

        self.testLabel = QLabel("Test")
        self.testLabel.setAlignment(Qt.AlignCenter)

        self.test = QComboBox()
        self.test.addItems(["Test 1", "Test 2"])

        self.topLayout = QHBoxLayout()
        self.topLayout.addWidget(self.comPortLabel)
        self.topLayout.addWidget(self.com_port)
        self.topLayout.addWidget(self.testLabel)
        self.topLayout.addWidget(self.test)

        # Create the bottom part with the buttons and result
        self.execOneButton = QPushButton("Execute one test")
        self.execAllButton = QPushButton("Execute all tests")
        self.resultLabel = QLabel("Result")
        self.resultLabel.setAlignment(Qt.AlignCenter)

        self.bottomLayout = QHBoxLayout()
        self.bottomLayout.addWidget(self.execOneButton)
        self.bottomLayout.addWidget(self.execAllButton)
        self.bottomLayout.addWidget(self.resultLabel)

        # Add the top and bottom layouts to the main layout
        self.mainLayout.addLayout(self.topLayout)
        self.mainLayout.addLayout(self.bottomLayout)

        # Connect the buttons to the methods
        self.execOneButton.clicked.connect(self._exec_one_test)
        self.execAllButton.clicked.connect(self._exec_all_tests)

    def _get_available_ports(self):
        """
        Gets the available ports on the system.
        :return: A list of strings with the available ports.
        """
        if sys.platform.startswith('win'):
            ports = ['COM%s' % (i + 1) for i in range(256)]
        elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
            # this excludes your current terminal "/dev/tty"
            ports = glob.glob('/dev/tty[A-Za-z]*')
        elif sys.platform.startswith('darwin'):
            ports = glob.glob('/dev/tty.*')
        else:
            raise EnvironmentError('Unsupported platform')

        result = []
        for port in ports:
            try:
                s = serial.Serial(port)
                s.close()
                result.append(port)
            except (OSError, serial.SerialException):
                pass
        return result

    def _exec_one_test(self):
        """
        Executes one test.
        :return: None.
        """
        self._exec_test(self.test.currentText())

    def _exec_all_tests(self):
        """
        Executes all tests.
        :return: None.
        """
        for test in self.test.currentText():
            self._exec_test(test)

    def _exec_test(self, test):
        """
        Executes a test.
        :param test: The name of the test to be executed.
        :return: None.
        """

        # Get the COM port from the dropdown list and open it with the baud rate 9600bps, no parity and 1 stop bit.
        com = self.com_port.currentText()
        ser = serial.Serial(com, 9600, serial.EIGHTBITS, serial.PARITY_NONE, serial.STOPBITS_ONE)

        # Send the test name to the serial port and wait for 1 second to receive the result.
        ser.write(test.encode())
        time.sleep(1)

        # Read the result from the serial port and show it on the label.
        result = ser.readline().decode()
        self.resultLabel.setText(result)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    mainWindow = MainWindow()
    mainWindow.show()
    sys.exit(app.exec_())